package com.ricerocks.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.InputProcessor;
import com.badlogic.gdx.Screen;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.SpriteBatch;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.ricerocks.RiceRocks;

public class AbstractScreen implements Screen, InputProcessor {

	protected static final float GAME_WIDTH = 800f;
	protected static final float GAME_HEIGHT = 480f;
	protected final RiceRocks game;
	protected final BitmapFont font;
	protected final SpriteBatch batch;
	protected final Stage stage;
	protected OrthographicCamera camera;

	public AbstractScreen(RiceRocks game) {
		this.game = game;
		this.font = new BitmapFont();
		this.batch = new SpriteBatch();
		this.stage = new Stage(0, 0, true);
		this.camera = new OrthographicCamera();
		this.camera.setToOrtho(false, GAME_WIDTH, GAME_HEIGHT);
		Gdx.input.setInputProcessor(this);
	}

	protected String getName() {
		return getClass().getSimpleName();
	}

	// Screen implementation
	@Override
	public void show() {
		Gdx.app.log(RiceRocks.LOG, "Showing screen: " + getName());
	}

	@Override
	public void resize(int width, int height) {
		Gdx.app.log(RiceRocks.LOG, "Resizing screen: " + getName() + " to: " + width + " x " + height);

		// resize the stage
		stage.setViewport(width, height, true);
	}

	@Override
	public void render(float delta) {
		// the following code clears the screen with the given RGB color (black)
		Gdx.gl.glClearColor(0f, 0f, 0f, 1f);
		Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT);

		// update and draw the stage actors
		camera.update();
		batch.setProjectionMatrix(camera.combined);
		stage.act(delta);
		stage.draw();
	}

	@Override
	public void hide() {
		Gdx.app.log(RiceRocks.LOG, "Hiding screen: " + getName());
	}

	@Override
	public void pause() {
		Gdx.app.log(RiceRocks.LOG, "Pausing screen: " + getName());
	}

	@Override
	public void resume() {
		Gdx.app.log(RiceRocks.LOG, "Resuming screen: " + getName());
	}

	@Override
	public void dispose() {
		Gdx.app.log(RiceRocks.LOG, "Disposing screen: " + getName());

		// dispose the collaborators
		stage.dispose();
		batch.dispose();
		font.dispose();
	}

	@Override
	public boolean keyDown(int keycode) {
		return false;
	}

	@Override
	public boolean keyUp(int keycode) {
		return false;
	}

	@Override
	public boolean keyTyped(char character) {
		return false;
	}

	@Override
	public boolean touchDown(int x, int y, int pointer, int button) {
		return false;
	}

	@Override
	public boolean touchUp(int x, int y, int pointer, int button) {
		return false;
	}

	@Override
	public boolean touchDragged(int x, int y, int pointer) {
		return false;
	}

	@Override
	public boolean touchMoved(int x, int y) {
		return false;
	}

	@Override
	public boolean scrolled(int amount) {
		return false;
	}
}
