package com.ricerocks.screens;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.Animation;
import com.badlogic.gdx.graphics.g2d.Sprite;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.MathUtils;
import com.badlogic.gdx.math.Rectangle;
import com.badlogic.gdx.math.Vector3;
import com.badlogic.gdx.utils.Array;
import com.ricerocks.Missile;
import com.ricerocks.RiceRocks;
import com.ricerocks.Rock;
import com.ricerocks.audio.GameSound;

public class PlayScreen extends AbstractScreen {

	private Animation exploAnimation, exploAnimation1;
	private Texture shipImage, rockImage, padImage, backgroundImage, debrisImage, fireImage, bulletImg, exploSheet, exploSheet1;
	private TextureRegion[] shipFrames, exploFrames, exploFrames1;
	private TextureRegion currentFrame, exploCurrent, exploCurrent1;
	private Rectangle ship, pad, debris;
	private Array<Rock> rocksss;
	private Array<Missile> bomb;
	private int rockCount = 8, live = 3, score = 0;
	private float debrisVelXY = 0.4f, shipAngel = 90f, velX = 0.0f, velY = 0.0f, maxSpeed = 3f, stateTime, stateTime1, exploX = -100f, exploY = -100f, exploX1 = -100f, exploY1 = -100f;
	private long lastTouchedTime, lastSpawntime, timeSpawnRock, timeReacttack;
	private GameSound sound;
	private float volume = 0.2f;

	public PlayScreen(RiceRocks game) {
		super(game);
	}

	@Override
	public void resize(int width, int height) {
		super.resize(width, height);
		stage.clear();
	}

	@Override
	public void show() {
		super.show();

		// SOUND
		sound = new GameSound();
		sound.soundTrack.setLooping(true);
		sound.soundTrack.setVolume(volume);
		sound.soundTrack.play();

		// SHIP IMAGE AND RECTANGLE
		shipImage = new Texture("data/double_ship.png");
		ship = new Rectangle();
		ship.width = shipImage.getWidth() / 2;
		ship.height = shipImage.getHeight();
		ship.x = GAME_WIDTH / 2 - ship.width / 2;
		ship.y = ship.height / 2;

		TextureRegion[][] tmp = TextureRegion.split(shipImage, shipImage.getWidth() / 2, shipImage.getHeight());
		shipFrames = new TextureRegion[2];
		shipFrames[0] = tmp[0][0];
		shipFrames[1] = tmp[0][1];
		currentFrame = shipFrames[0];

		// CONTROLLER
		padImage = new Texture("data/pad.png");
		pad = new Rectangle();
		pad.width = padImage.getWidth();
		pad.height = padImage.getHeight();
		pad.x = 10;
		pad.y = 10;

		// ROCK, BACKGROUND, FIRE BUTTON, MISSILE
		rockImage = new Texture("data/asteroid_blue.png");
		backgroundImage = new Texture("data/nebula_blue.png");
		fireImage = new Texture("data/fire.png");
		bulletImg = new Texture("data/shot1.png");

		// EXPLOSION
		exploSheet = new Texture("data/explosion_blue2.png");
		tmp = TextureRegion.split(exploSheet, exploSheet.getWidth() / 24, exploSheet.getHeight());
		exploFrames = new TextureRegion[24];
		for (int i = 0; i < 24; i++) {
			exploFrames[i] = tmp[0][i];
		}
		exploAnimation = new Animation(0.015f, exploFrames);
		stateTime = 0f;

		exploSheet1 = new Texture("data/explosion_orange.png");
		tmp = TextureRegion.split(exploSheet1, exploSheet1.getWidth() / 24, exploSheet1.getHeight());
		exploFrames1 = new TextureRegion[24];
		for (int i = 0; i < 24; i++) {
			exploFrames1[i] = tmp[0][i];
		}
		exploAnimation1 = new Animation(0.025f, exploFrames1);
		stateTime1 = 0f;

		// DYNAMIC BACKGROUND
		debrisImage = new Texture("data/debris3_blue.png");
		debris = new Rectangle();
		debris.width = debrisImage.getWidth();
		debris.height = debrisImage.getHeight();
		debris.x = -debris.width / 2;
		debris.y = GAME_HEIGHT;

		rocksss = new Array<Rock>();
		bomb = new Array<Missile>();
	}

	@Override
	public void render(float delta) {
		super.render(delta);

		try {
			Color c = batch.getColor();
			stateTime += Gdx.graphics.getDeltaTime();
			exploCurrent = exploAnimation.getKeyFrame(stateTime, false);

			stateTime1 += Gdx.graphics.getDeltaTime();
			exploCurrent1 = exploAnimation1.getKeyFrame(stateTime1, false);

			batch.begin();
			batch.setColor(c.r, c.g, c.b, 1);
			batch.draw(backgroundImage, 0, 0, GAME_WIDTH, GAME_HEIGHT);
			batch.draw(debrisImage, debris.x, debris.y);

			Sprite rock;
			Rectangle recTmp;
			Missile misTmp;
			for (int i = 0; i < rocksss.size; i++) {
				rock = rocksss.get(i).rockSprite; // sprite
				recTmp = rocksss.get(i).rockRectangle; // rectangle
				rock.rotate(rocksss.get(i).rotation); // set rotation
				// update rectangle
				recTmp.x = rock.getX() + rocksss.get(i).rockVelX;
				recTmp.y = rock.getY() + rocksss.get(i).rockVelY;
				// update sprite move
				rock.setPosition(rock.getX() + rocksss.get(i).rockVelX, rock.getY() - rocksss.get(i).rockVelY);
				rock.draw(batch, 1);
			}
			batch.setColor(c.r, c.g, c.b, 1);
			for (int i = 0; i < bomb.size; i++) {
				misTmp = bomb.get(i);
				misTmp.misslite.x += Float.parseFloat(13 * Math.cos(Math.toRadians(misTmp.angle)) + "");
				misTmp.misslite.y += Float.parseFloat(13 * Math.sin(Math.toRadians(misTmp.angle)) + "");

				batch.draw(misTmp.missliteImage, misTmp.misslite.x, misTmp.misslite.y, misTmp.misslite.width / 2, misTmp.misslite.height / 2, misTmp.misslite.width, misTmp.misslite.height, 1f, 1f, misTmp.angle);
			}
			batch.draw(currentFrame, ship.x, ship.y, ship.height / 2, ship.height / 2, ship.width, ship.height, 1f, 1f, shipAngel);
			batch.draw(exploCurrent, exploX, exploY);
			batch.draw(exploCurrent1, exploX1, exploY1);

			batch.draw(padImage, pad.x, pad.y);
			batch.setColor(c.r, c.g, c.b, 0.6f);
			batch.draw(fireImage, GAME_WIDTH - fireImage.getWidth() - 10, 10);
			font.setColor(Color.RED);
			font.draw(batch, "LIVES: " + live, 10, GAME_HEIGHT - 10);
			font.draw(batch, "SCORE: " + score, 10, GAME_HEIGHT - 30);
			batch.end();

			// set debris move
			if (debris.x > GAME_WIDTH)
				debris.x = -debris.width;
			else
				debris.x += debrisVelXY;

			if (debris.y + debris.height < 0)
				debris.y = GAME_HEIGHT;
			else
				debris.y -= debrisVelXY;

			// remove rock outsite screen
			for (int i = 0; i < rocksss.size; i++) {
				rock = rocksss.get(i).rockSprite;
				if (rock.getX() > GAME_WIDTH || rock.getY() + rock.getHeight() < 0) {
					rocksss.removeIndex(i);
				}
			}
			// remove missile
			for (int i = 0; i < bomb.size; i++) {
				if (bomb.get(i).misslite.x > GAME_WIDTH || bomb.get(i).misslite.y > GAME_HEIGHT) {
					bomb.removeIndex(i);
				}
			}

			// keep the ship inside sceen
			if (ship.x + ship.width < 0)
				ship.x = GAME_WIDTH;
			else if (ship.x > GAME_WIDTH)
				ship.x = -ship.width;
			else
				ship.x += velX;

			if (ship.y + ship.width < 0)
				ship.y = GAME_HEIGHT;
			else if (ship.y > GAME_HEIGHT)
				ship.y = -ship.height;
			else
				ship.y += velY;

			// process user input
			if (live > 0) {
				// KEY PRESS
				if (Gdx.input.isKeyPressed(Keys.SPACE)) {
					if (System.nanoTime() - timeReacttack > 200000000) {
						sound.missile.play(volume);
						Missile bullet = new Missile();
						bullet.missliteImage = new TextureRegion(bulletImg);
						bullet.misslite = new Rectangle();
						bullet.misslite.width = bulletImg.getWidth();
						bullet.misslite.height = bulletImg.getHeight();
						bullet.misslite.x = ship.x + (ship.width - bullet.misslite.width) / 2;
						bullet.misslite.y = ship.y + (ship.width - bullet.misslite.height) / 2;
						bullet.angle = shipAngel;
						bomb.add(bullet);
						timeReacttack = System.nanoTime();
					}
				}
				if (Gdx.input.isKeyPressed(Keys.UP)) {
					if (Math.abs(velX) < maxSpeed)
						velX += .1 * Float.parseFloat(Math.cos(Math.toRadians(shipAngel)) + "");
					if (Math.abs(velY) < maxSpeed)
						velY += .1 * Float.parseFloat(Math.sin(Math.toRadians(shipAngel)) + "");
				}
				if (Gdx.input.isKeyPressed(Keys.LEFT)) {
					shipAngel = (shipAngel + 3) % 360;
				}
				if (Gdx.input.isKeyPressed(Keys.RIGHT)) {
					shipAngel = (shipAngel - 3) % 360;
				}
				if (Gdx.input.isKeyPressed(Keys.UP) || Gdx.input.isKeyPressed(Keys.LEFT) || Gdx.input.isKeyPressed(Keys.RIGHT)) {
					lastTouchedTime = System.nanoTime();
					currentFrame = shipFrames[1];
				} else {
					currentFrame = shipFrames[0];
				}

				// TOUCH
				for (int i = 0; i < 2; i++) {
					if (Gdx.input.isTouched(i)) {
						Vector3 touchPos = new Vector3();
						touchPos.set(Gdx.input.getX(i), Gdx.input.getY(i), 0);
						camera.unproject(touchPos);

						if (distance2point(GAME_WIDTH - 10 - fireImage.getWidth() / 2, fireImage.getHeight() / 2 + 10, touchPos.x, touchPos.y) < fireImage.getWidth() / 2) {
							if (System.nanoTime() - timeReacttack > 200000000) {
								sound.missile.play(volume);
								Missile bullet = new Missile();
								bullet.missliteImage = new TextureRegion(bulletImg);
								bullet.misslite = new Rectangle();
								bullet.misslite.width = bulletImg.getWidth();
								bullet.misslite.height = bulletImg.getHeight();
								bullet.misslite.x = ship.x + (ship.width - bullet.misslite.width) / 2;
								bullet.misslite.y = ship.y + (ship.width - bullet.misslite.height) / 2;
								bullet.angle = shipAngel;
								bomb.add(bullet);
								timeReacttack = System.nanoTime();
							}
						}

						if (distance2point(touchPos.x, touchPos.y, pad.x + pad.width / 2, pad.y + pad.height / 2) <= pad.width / 2) {
							lastTouchedTime = System.nanoTime();
							if (distance2point(touchPos.x, touchPos.y, pad.x + pad.width / 2, pad.y + pad.height / 2 + pad.height / 4) <= pad.width / 4) {
								currentFrame = shipFrames[1];
								if (Math.abs(velX) < maxSpeed)
									velX += .1 * Float.parseFloat(Math.cos(Math.toRadians(shipAngel)) + "");
								if (Math.abs(velY) < maxSpeed)
									velY += .1 * Float.parseFloat(Math.sin(Math.toRadians(shipAngel)) + "");
							}
							// di lui
							if (distance2point(touchPos.x, touchPos.y, pad.x + pad.width / 2, pad.y + pad.height / 4) <= pad.width / 4) {
							}
							// sang phai
							if (distance2point(touchPos.x, touchPos.y, pad.x + pad.width / 2 + pad.width / 4, pad.y + pad.height / 2) <= pad.width / 4) {
								currentFrame = shipFrames[1];
								shipAngel = (shipAngel - 3) % 360;
							}
							// sang trai
							if (distance2point(touchPos.x, touchPos.y, pad.x + pad.width / 4, pad.y + pad.height / 2) <= pad.width / 4) {
								currentFrame = shipFrames[1];
								shipAngel = (shipAngel + 3) % 360;
							}
						}
					}
				}
			}

			// tao luc quan tinh' cho tau
			if (System.nanoTime() - lastTouchedTime > 200000000) {
				if (velX > 0) {
					velX -= Math.abs(Float.parseFloat(Math.cos(Math.toRadians(shipAngel)) + ""));
					if (velX < 0) {
						velX = 0;
						velY = 0;
					}
				}
				if (velX < 0) {
					velX += Math.abs(Float.parseFloat(Math.cos(Math.toRadians(shipAngel)) + ""));
					if (velX > 0) {
						velX = 0;
						velY = 0;
					}
				}

				if (velY > 0) {
					velY -= Math.abs(Float.parseFloat(Math.sin(Math.toRadians(shipAngel)) + ""));
					if (velY < 0) {
						velX = 0;
						velY = 0;
					}
				}
				if (velY < 0) {
					velY += Math.abs(Float.parseFloat(Math.sin(Math.toRadians(shipAngel)) + ""));
					if (velY > 0) {
						velX = 0;
						velY = 0;
					}
				}

				lastTouchedTime = System.nanoTime();
			}

			// check if we need to create a new rock
			if (System.nanoTime() - lastSpawntime > timeSpawnRock)
				spawnRock();

			// overlaps
			for (int i = 0; i < rocksss.size; i++) {
				Rectangle rr = rocksss.get(i).rockRectangle;
				if (rr.overlaps(ship)) {
					if (distance2point(rr.x + rr.width / 2, rr.y + rr.height / 2, ship.x + ship.width / 2, ship.y + ship.height / 2) <= ship.width - 20) {
						stateTime = 0f;
						exploX = rr.x - rr.width / 4;
						exploY = rr.y - rr.height / 2;
						rocksss.removeIndex(i);
						sound.explosion.play(volume);
						live--;
						if (live == 0) {
							timeReacttack = System.nanoTime();
							stateTime1 = 0f;
							exploX1 = ship.x - ship.height / 4;
							exploY1 = ship.y - ship.height / 4;

							rockCount = 0;
							ship.width = 0;
							ship.height = 0;
							sound.soundTrack.stop();
							sound.die.play(volume);
						}
					}
				}
				for (int j = 0; j < bomb.size; j++) {
					if (rr.overlaps(bomb.get(j).misslite)) {
						stateTime = 0f;
						exploX = rr.x - rr.width / 4;
						exploY = rr.y - rr.height / 2;
						bomb.removeIndex(j);
						rocksss.removeIndex(i);
						sound.explosion.play(volume);
						score += 10;
						if (score % 300 == 0) {
							live++;
						}
					}
				}
			}
			if (System.nanoTime() - timeReacttack > 2147482999 && live == 0)
				game.setScreen(game.getMenuScreen());
		} catch (Exception ex) {
		}
	}

	@Override
	public boolean touchUp(int x, int y, int pointer, int button) {
		currentFrame = shipFrames[0];
		return false;
	}

	@Override
	public void dispose() {
		super.dispose();

		shipImage.dispose();
		rockImage.dispose();
		padImage.dispose();
		backgroundImage.dispose();
		debrisImage.dispose();
		fireImage.dispose();
		bulletImg.dispose();
		exploSheet.dispose();

		sound.explosion.stop();
		sound.missile.stop();
		sound.soundTrack.stop();
		sound.die.stop();

		sound.explosion.dispose();
		sound.missile.dispose();
		sound.soundTrack.dispose();
		sound.die.dispose();
	}

	private float distance2point(float x1, float y1, float x2, float y2) {
		float rs = 0;
		rs = (x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2);
		rs = (float) Math.sqrt(Double.parseDouble(rs + ""));

		return rs;
	}

	private void spawnRock() {
		if (rocksss.size < rockCount) {
			Sprite rockSpr = new Sprite(rockImage);
			Rectangle rockRec = new Rectangle();
			Rock r = new Rock();

			rockRec.width = rockSpr.getWidth();
			rockRec.height = rockSpr.getHeight();
			rockRec.x = MathUtils.random(0, GAME_WIDTH - rockRec.width);
			rockRec.y = GAME_HEIGHT - 1;

			rockSpr.setPosition(rockRec.x, rockRec.y);

			r.rockRectangle = rockRec;
			r.rockSprite = rockSpr;
			r.rotation = MathUtils.random(-5, 5);
			r.rockVelX = MathUtils.random(0, 1);
			r.rockVelY = MathUtils.random(1, 2);
			rocksss.add(r);

			lastSpawntime = System.nanoTime();
		}
	}
}
